----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

photoMode = {};

function photoMode:getUILanguage(lang)
	local de = "de";
	if string.lower(lang) == de then
		return de;
	end;
	return "en";
end;

function initializePhotoMode()
	photoMode.isActive = false;
	photoMode.isSet = false;
	photoMode.invertRotY = false;
	photoMode.previousCamera = 0;
	photoMode.photoCamera = 0;
	photoMode.captureCoolDown = 0;
	--photoMode.lightCoolDown = 0;
	--photoMode.lightsource = g_currentModDirectory.."light.i3d";
	photoMode.captureSound = createSample("captureSound");
	loadSample(photoMode.captureSound, g_currentModDirectory.."cameraCapture.ogg", false);
	photoMode.warningText = Utils.getNoNil(asciiToUtf8(g_i18n:getText("photoMode_warningText")),"Exit vehicle for proper controls");
	local languageAbrv = photoMode:getUILanguage(g_languageShort);
	photoMode.controls = {};
	photoMode.controls.c0 = createImageOverlay(g_currentModDirectory.."images/controls0_"..languageAbrv..".dds");
	local wAdj = (16/9)/g_screenAspectRatio;
	local width,height = 0.15625*wAdj,0.09259;
	photoMode.controls.c0X, photoMode.controls.c0Y, photoMode.controls.c0W, photoMode.controls.c0H = 0.5-(width/2),0,width,height;
	photoMode.controls.c1 = createImageOverlay(g_currentModDirectory.."images/controls1_"..languageAbrv..".dds");
	width,height = 0.078125*wAdj,0.324074;
	photoMode.controls.c1X, photoMode.controls.c1Y, photoMode.controls.c1W, photoMode.controls.c1H = 0,0,width,height;
	photoMode.controls.c2 = createImageOverlay(g_currentModDirectory.."images/controls2_"..languageAbrv..".dds");
	width,height = 0.078125*wAdj,0.27778;
	photoMode.controls.c2X, photoMode.controls.c2Y, photoMode.controls.c2W, photoMode.controls.c2H = 1-width,0,width,height;
	GamePausedDisplay.draw = Utils.overwrittenFunction(GamePausedDisplay.draw, photoMode.nil_draw);
	GamePausedDisplay.createBackground = Utils.overwrittenFunction(GamePausedDisplay.createBackground, photoMode.nil_createBackground);
end;

function photoMode:moveCam(cam)
	local speed = 0.1;
	local rotspeed = 0.015;
	if Input.isKeyPressed(Input.KEY_lshift) then
		speed = speed*10;
		rotspeed = rotspeed*10;
	end;

	local x,y,z = getRotation(cam);
	if Input.isKeyPressed(Input.KEY_KP_4) then
		if photoMode.invertRotY then setRotation(cam, x,y-rotspeed,z);
		else setRotation(cam, x,y+rotspeed,z); end;
	elseif Input.isKeyPressed(Input.KEY_KP_6) then
		if photoMode.invertRotY then setRotation(cam, x,y+rotspeed,z);
		else setRotation(cam, x,y-rotspeed,z); end;
	elseif Input.isKeyPressed(Input.KEY_KP_8) then
		setRotation(cam, x+rotspeed,y,z);
	elseif Input.isKeyPressed(Input.KEY_KP_5) then
		setRotation(cam, x-rotspeed,y,z);
	elseif Input.isKeyPressed(Input.KEY_KP_7) then
		x,y,z = getRotation(getChildAt(cam,0));
		setRotation(getChildAt(cam,0), 0,0,z-rotspeed);
	elseif Input.isKeyPressed(Input.KEY_KP_9) then
		x,y,z = getRotation(getChildAt(cam,0));
		setRotation(getChildAt(cam,0), 0,0,z+rotspeed);
	elseif Input.isKeyPressed(Input.KEY_KP_0) then
		setRotation(getChildAt(cam,0), 0,0,0);
	end;

	x,y,z = getTranslation(cam);
	if Input.isKeyPressed(Input.KEY_up) then
		local x2,y2,z2 = localDirectionToWorld(cam,0,1,0);
		setTranslation(cam, x+(speed*x2),y+(speed*y2),z+(speed*z2));
	elseif Input.isKeyPressed(Input.KEY_down) then
		local x2,y2,z2 = localDirectionToWorld(cam,0,-1,0);
		setTranslation(cam, x+(speed*x2),y+(speed*y2),z+(speed*z2));
	elseif Input.isKeyPressed(Input.KEY_a) then
		local x2,y2,z2 = localDirectionToWorld(cam,-1,0,0);
		setTranslation(cam, x+(speed*x2),y+(speed*y2),z+(speed*z2));
	elseif Input.isKeyPressed(Input.KEY_d) then
		local x2,y2,z2 = localDirectionToWorld(cam,1,0,0);
		setTranslation(cam, x+(speed*x2),y+(speed*y2),z+(speed*z2));
	elseif Input.isKeyPressed(Input.KEY_w) then
		local x2,y2,z2 = localDirectionToWorld(cam,0,0,-1);
		setTranslation(cam, x+(speed*x2),y+(speed*y2),z+(speed*z2));
	elseif Input.isKeyPressed(Input.KEY_s) then
		local x2,y2,z2 = localDirectionToWorld(cam,0,0,1);
		setTranslation(cam, x+(speed*x2),y+(speed*y2),z+(speed*z2));
	end;
end;

function photoMode:isCameraInside()
	if g_currentMission.controlledVehicle ~= nil then
		if g_currentMission.controlledVehicle.getActiveCamera ~= nil then
			local activeCamera = g_currentMission.controlledVehicle:getActiveCamera();
			if activeCamera ~= nil and activeCamera.isInside then
				return true;
			end;
		end;
	end;
	return false;
end;

function photoMode:displayControls(controls)
	if photoMode.captureCoolDown < 1 then
		renderOverlay(controls.c0, controls.c0X, controls.c0Y, controls.c0W, controls.c0H);
		renderOverlay(controls.c1, controls.c1X, controls.c1Y, controls.c1W, controls.c1H);
		renderOverlay(controls.c2, controls.c2X, controls.c2Y, controls.c2W, controls.c2H);

		if photoMode:isCameraInside() then
			local textSize = 0.04*g_gameSettings:getValue("uiScale");
			local oX = textSize * HUDTextDisplay.SHADOW_OFFSET_FACTOR;
			setTextAlignment(RenderText.ALIGN_CENTER);
			setTextColor(0,0,0,1);
			renderText(0.5+oX,0.1-oX,textSize,photoMode.warningText);
			setTextColor(1,0.5,0,1);
			renderText(0.5,0.1,textSize,photoMode.warningText);
        end;
	end;
end;

function photoMode:captureScreen()
	local photoModeScreenshotDir = g_screenshotsDirectory.."PhotoMode";
	createFolder(photoModeScreenshotDir);
	local screenshotName = photoModeScreenshotDir.."/fsScreen_"..getDate("%Y_%m_%d_%H_%M_%S").."_PhotoMode.png";
	print("Saving screenshot: "..screenshotName);
	playSample(photoMode.captureSound,1,0.8,0,0,0);
	saveScreenshot(screenshotName);
end;

function photoMode:inputUpdate()
	if Input.isKeyPressed(Input.KEY_return) and photoMode.captureCoolDown < 1 then
		photoMode.captureCoolDown = 40;
	end;
	if photoMode.captureCoolDown > 0 then
		if photoMode.captureCoolDown == 35 then
			photoMode:captureScreen();
		end;
		photoMode.captureCoolDown = photoMode.captureCoolDown - 1;
	end;
	--[[if Input.isKeyPressed(Input.KEY_l) and photoMode.lightCoolDown < 1 then
		if getVisibility(photoMode.lightNode) then
			setVisibility(photoMode.lightNode,false);
		else
			setVisibility(photoMode.lightNode,true);print("on");
		end;
		photoMode.lightCoolDown = 15;
	end;
	if photoMode.lightCoolDown > 0 then
		photoMode.lightCoolDown = photoMode.lightCoolDown - 1;
	end;]]
	photoMode:moveCam(photoMode.photoCamera);
end;

function photoMode:nil_draw(superFunc,dt)
	photoMode.isActive = self:getVisible();
	if photoMode.isActive then
		if not photoMode.isSet then
			photoMode.previousCamera = getCamera();
			photoMode.photoCamera = createTransformGroup("photoCam");
			local photoCam = createCamera("photoCam", getFovY(photoMode.previousCamera), 0.1, 5000);
			--photoMode.lightNode = loadI3DFile(photoMode.lightsource,false,false,false);
			--photoMode.lightNode = getChildAt(photoMode.lightNode,0);
			link(photoMode.photoCamera, photoCam);
			--link(photoMode.photoCamera, photoMode.lightNode);
			local x,y,z = getWorldTranslation(photoMode.previousCamera);
			setWorldTranslation(photoMode.photoCamera, x,y,z);
			x,y,z = getWorldRotation(photoMode.previousCamera);
			setWorldRotation(photoMode.photoCamera, x,y,z);
			--setVisibility(photoMode.lightNode,false);
			setCamera(photoCam);
			photoMode.isSet = true;
			if z < 0 then photoMode.invertRotY = true;
			else photoMode.invertRotY = false; end;
		end;

		photoMode:displayControls(photoMode.controls);
		photoMode:inputUpdate();
	end;
end;

function photoMode:nil_createBackground(superFunc,hudAtlasPath)
    local _, height = getNormalizedScreenValues(unpack(GamePausedDisplay.SIZE.SELF));
    local overlay = Overlay:new(hudAtlasPath, 0, (1 - height) * 0.5, 1, height);
    overlay:setUVs(getNormalizedUVs(GamePausedDisplay.UV.BACKGROUND));
    overlay:setColor(0,0,0,0);
    return overlay;
end;

function photoMode:update(dt)
	if photoMode.isActive then
		setCamera(photoMode.previousCamera);
		delete(photoMode.photoCamera);
		photoMode.previousCamera = 0;
		photoMode.captureCoolDown = 0;
		photoMode.isSet = false;
		photoMode.isActive = false;
	end;
end;

initializePhotoMode();
addModEventListener(photoMode);